--this has our functions and things to make our entity system work
--this is basically the entity system!
ents = {}

ents.objects = {} --a table within the ents table of all the active objects

ents.objectPath = "entities/" --just a string that is the path to the entities folder

local register = {} --a table that has the different paths the the entites that we can call

--each entity will get an ID
local id = 0
--this function  in the ents table is called when the game starts and will load and put all of our templates into the registry so we can use them
--when we make a new entity template we put it in here
function ents.Startup()
    register["ball"] = love.filesystem.load(ents.objectPath .. "ball.lua")
    register["paddle"] = love.filesystem.load(ents.objectPath .. "paddle.lua")
end

--will load the file that we pass into it. ie: ball.lua
function ents.Derive(name)
    return love.filesystem.load(ents.objectPath .. name .. ".lua")()
end

--when we create a new entitiy, it will look for the entitiy template in the register, make a new ID for it, run the functions to spawn it
    --then put it in the active objects table
function ents.Create(name, x, y)
    if register[name] then
        id = id + 1
        local ent = register[name]()
        ent:load() -- call the load function in the specific entity
        ent.type = name --a new variable that is just a string of the type of the thing, block, ball etc
        ent:setPos(x, y)
        ent.id = id
        ents.objects[#ents.objects + 1] = ent
        return ents.objects[#ents.objects]
    else
        print("Entity is not in the registry")
        return false;
    end
end

--these functions go through the active objects table and do the thing to the individual object when it is called
--when we pass the function "id" we are telling the function which one to destroy
function ents.Destroy(id)
    if ents.objects[id] then  --if the active entity exists on the screen
        if ents.objects[id].Die then --check if the Entity has the Die function
            ents.objects[id]:Die() --if it does, call the Die function
        end
        ents.objects[id] = nil --set the value in the objects table to nil (remove it from the table), it is no longer an active object
    end
end

--We also need to be able to use the update function INSIDE THE SPECIFIC entity, so that it doesn't affect everything
--each time this is run (which will be put into out main update function) in main.lua...
function ents:update(dt)
    for i, ent in pairs(ents.objects) do --...it goes through every object in the ents.objects table (which again, is all of our active entities on the screen)
        if ent.update then --Checks if it has an ent.update function (it will be in our file for the entity and just called update in the file)
            ent:update(dt) --if it does, run it with the vaule dt, it is like calling it in love.update
        end
    end
end

--This function is very smilar to the one above, but instead it is draw
function ents:draw()
    for i, ent in pairs(ents.objects) do --...it goes through all of the entities on the screen
        if ent.draw then --Checks if it has a draw function
            ent:draw() --if it does.. run it!
        end
    end
end
